(* Kopieren einer Datei in alle Unterverzeichnisse eines Basisverzeichnisses,
   (fr Testzwecke)

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   J. Rathlev, Jan. 2008
   last changes: 
   *)

unit MuCoMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, HListBox, XPMan, ComCtrls;

const
  ProgName = 'Multi copy of file';
  Vers = '1.1';
  CopRgt = ' 2008 Dr. J. Rathlev, 24222 Schwentinental';
  EmailAdr = 'info(a)rathlev-home.de';


type
  THauptForm = class(TForm)
    Label3: TLabel;
    SourceFileEdit: THistoryCombo;
    FileBtn: TSpeedButton;
    FormDirBtn: TSpeedButton;
    RootDirEdit: THistoryCombo;
    Label1: TLabel;
    btnCopy: TSpeedButton;
    InfoBtn: TSpeedButton;
    EndeBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    Label2: TLabel;
    LogBox: TListBox;
    FolderIdEdit: THistoryCombo;
    Label4: TLabel;
    XPManifest1: TXPManifest;
    StatusBar: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure FormDirBtnClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    AppPath,UserPath,
    IniName          : string;
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.dfm}

uses JrUtils, ShellDirDlg, ShlObj, StrUtils, WinDesktop, WinApiUtils,
  FileUtils, GnuGetText, LangUtils;

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  SourceSekt = 'Source';
  DestSekt = 'Dest';
  FileSekt = 'Files';
  IdSekt = 'SubFolders';

  (* INI-Variablen *)

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
begin
  TranslateComponent(self);
  InitPaths(AppPath,UserPath);
  InitVersion(ProgName,Vers,CopRgt,3,3,ProgVersName,ProgVersDate);
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  RootDirEdit.LoadFromIni (IniName,SourceSekt);
  RootDirEdit.ItemIndex:=0;
  SourceFileEdit.LoadFromIni (IniName,DestSekt);
  SourceFileEdit.ItemIndex:=0;
  FolderIdEdit.LoadFromIni(IniName,IdSekt);
  FolderIdEdit.ItemIndex:=0;
  Caption:=ProgVersName;
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog (ProgName,_('Multiple copies of a file')+' - '
           +ProgVersDate+sLineBreak+CopRgt
           +sLineBreak+'E-Mail: '+EmailAdr,CenterPos);
  end;

procedure THauptForm.FileBtnClick(Sender: TObject);
begin
  with OpenDialog do begin
    if length(SourceFileEdit.Text)>0 then InitialDir:=ExtractFilePath(SourceFileEdit.Text)
    else InitialDir:=UserPath;
    Filename:=ExtractFileName(SourceFileEdit.Text);
    Filter:=_('All files')+'|*.*';
    Title:=_('Select source file');
    if Execute then with SourceFileEdit do begin
      Text:=Filename; AddItem(Filename);
      end;
    end;
  end;

procedure THauptForm.FormDirBtnClick(Sender: TObject);
var
  s : string;
begin
  s:=RootDirEdit.Text;
  if length(s)=0 then s:=UserPath;
  if ShellDirDialog.Execute (_('Select root directory'),false,true,'',s) then begin
    with RootDirEdit do begin
      Text:=s; AddItem(s);
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.btnCopyClick(Sender: TObject);
var
  fcnt : integer;

  procedure CopyToDir (const Dir,FolderId,Filename : string);
  var
    DirInfo    : TSearchRec;
    Findresult : integer;
    s          : string;
  begin
    FindResult:=FindFirst (Erweiter(Dir,'*','*'),faAnyFile,DirInfo);
    while FindResult=0 do with DirInfo do begin
      if NotSpecialDir(Name) then begin
        if((Attr and faDirectory)<>0) then CopyToDir(Erweiter(Dir,Name,''),FolderId,Filename);
        end;
      FindResult:=FindNext (DirInfo);
      end;
    FindClose(DirInfo);
    if AnsiContainsText(Dir,FolderId) then begin
      s:=Erweiter(Dir,Filename,'');
      FindResult:=FindFirst (s,faArchive+faReadOnly+faHidden+faSysfile,DirInfo);
      while FindResult=0 do with DirInfo do begin
        if NotSpecialDir(Name) then begin
          try
          CopyFileTS(SourceFileEdit.Text,s);
            LogBox.Items.Add(s);
            inc(fcnt);
          except
            on E:EInOutError do begin
              ErrorDialog('',E.Message);
              LogBox.Items.Add(_('==> Error: ')+s);
              end;
            end;
          end;
        FindResult:=FindNext (DirInfo);
        end;
      FindClose(DirInfo);
      end;
    end;

begin
  with SourceFileEdit do AddItem(Text);
  with RootDirEdit do AddItem(Text);
  with FolderIdEdit do AddItem(Text);
  LogBox.Clear;
  fcnt:=0;
  if FileExists(SourceFileEdit.Text) then begin
    CopyToDir(RootDirEdit.Text,
      IncludeTrailingPathDelimiter('')+IncludeTrailingPathDelimiter(FolderIdEdit.Text),
      ExtractfileName(SourceFileEdit.Text));
    with Statusbar do begin
      if fcnt=0 then SimpleText:=_('No matching destination found!')
      else SimpleText:=Format(_('File was copied %u times'),[fcnt]);
      end;
    end
  else begin
    ErrorDialog('',Format(_('File not found:')+sLineBreak+'%s',[SourceFileEdit.Text]));
    Statusbar.SimpleText:=_('Source file not found');
    end;
  end;

end.
